package org.nuiton.eugene.models.object.reader.yaml;

/*
 * #%L
 * EUGene :: EUGene
 * $Id: YamlToObjectModel.java 1262 2013-05-29 10:06:05Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/eugene/tags/eugene-2.6.3/eugene/src/main/java/org/nuiton/eugene/models/object/reader/yaml/YamlToObjectModel.java $
 * %%
 * Copyright (C) 2004 - 2013 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

import org.nuiton.eugene.models.object.*;
import org.nuiton.eugene.models.object.xml.*;
import org.yaml.snakeyaml.Yaml;

import java.io.*;
import java.util.*;

import static org.nuiton.eugene.models.object.reader.yaml.YamlUtil.afterChar;
import static org.nuiton.eugene.models.object.reader.yaml.YamlUtil.browseKeySetToLowerCase;

/**
 * User: agiraudet
 * Date: 15/05/13
 * Time: 11:15
 */

public class YamlToObjectModel {
    protected Yaml yaml;
    protected String packageOM;
    protected Map<String, String> imports;
    protected Map<String, String> defaultValues;

    //mots clefs
    public static final String ABSTRACT = "abstract";
    public static final String ASSOCIATION_CLASS_NAME = "associationclassname";
    public static final String ASSOCIATION_TYPE = "associationtype";
    public static final String ATTRIBUTE = "attribute";
    public static final String BODY_CODE = "bodeycode";
    public static final String CLASS = "class";
    public static final String CLASS_ASSOCIATION = "associationclass";
    public static final String COMMENTS = "comments";
    public static final String DEFAULT_VALUE = "defaultvalue";
    public static final String DOCUMENTATION = "documentation";
    public static final String ENUMERATION = "enumeration";
    public static final String EXTERN = "extern";
    public static final String FINAL = "final";
    public static final String IMPORTS = "imports";
    public static final String INNER = "inner";
    public static final String INTERFACE = "interface";
    public static final String LITERAL = "literal";
    public static final String MAX_MULTIPLICITY = "maxmultiplicity";
    public static final String MIN_MULTIPLICITY = "minmultiplicity";
    public static final String NAME = "name";
    public static final String NAVIGABLE = "navigable";
    public static final String OPERATION = "operation";
    public static final String ORDERED = "ordered";
    public static final String ORDERING = "ordering";
    public static final String PACKAGE = "package";
    public static final String PARAMETER = "parameter";
    public static final String PARTICIPANT = "participant";
    public static final String RETURN_PARAMETER = "returnparameter";
    public static final String REVERSE_ATTRIBUTE_NAME = "reverseattributename";
    public static final String REVERSE_MAX_MULTIPLICITY = "reversemaxmultiplicity";
    public static final String STATIC = "static";
    public static final String STEREOTYPES = "stereotypes";
    public static final String SUPER_CLASSES = "superclasses";
    public static final String SUPER_INTERFACES = "superinterfaces";
    public static final String TAG_VALUES = "tagvalues";
    public static final String TRANSIENT = "transient";
    public static final String TYPE = "type";
    public static final String UNIQUE = "unique";
    public static final String VERSION = "version";
    public static final String VISIBILITY = "visibility";
    public static final String ELEMENT = "element";

    private boolean _loginit = true;
    private void log(String str) {
        try {
            FileWriter log;
            String path = "/tmp/log.yaml.txt";

            if (_loginit) {
                log = new FileWriter(path, false);
                log.close();
                _loginit = false;
            }
            log = new FileWriter(path, true);
            log.write("[LOG] " + str + "\n");
            log.close();
        } catch (IOException e) {;}
    }

    public YamlToObjectModel() {
        yaml = new Yaml();
    }

    //TODO: gestion des labels ? -> non pas pour le moment (plutot pour la syntaxe "user friendly")
    //TODO: retourner boolean sur reussite chargement ? ou lever excpetions ? -> aucun des deux pour le moment
    //TODO: charger valeurs par defaut en fonction de la version de la syntaxe yaml (syntaxe: version), charger les valeurs à partir d'un fichier ou d'une classe ? -> a partir d'un fichier YAML
    //TODO: mettre les valeurs par défaut en dur dans le code
    //TODOK: gérer les imports sous forme de List et récuperer le type
    //TODO: pre-charger les noms et paquetages des classes pour les imports
    //TODO: 1/ instancier les objets et les attributs de niveau 1 (name, package...) (et charger les imports vers les classes du modele) 2/ charger les objets de niveau 2 (attributes, operations...)
    /*
     * nouvelle implémentation : contraintes : labels, valeurs par défaut, parser des chaines (imports internes/externes)
     *
     */
    public void loadFile(File file, ObjectModel model) throws IOException {
        //TODO: valeurs par defaut, faire interface ? ou simple map ? + mots clefs ?
        defaultValues = new LinkedHashMap<String, String>();

        ObjectModelImpl modelOM = (ObjectModelImpl) model;
        InputStream inputModel = new FileInputStream(file);
        Object modelYAML = yaml.load(inputModel);
        inputModel.close();

        List<String> ignore = new LinkedList<String>();
        ignore.add(TAG_VALUES);
        browseKeySetToLowerCase(modelYAML, ignore);

        loadModel(modelYAML, modelOM);
    }

    public Object collectElement(List data, Object key) {
        for (Object obj : data) {
            if (obj instanceof Map) {
                if (((Map) obj).containsKey(key)) {
                    return ((Map) obj).get(key);
                }
            }
        }
        return null;
    }

    public List<Object> collectAllElements(List data, Object key) {
        List<Object> res = new LinkedList<Object>();
        for (Object obj : data) {
            if (obj instanceof Map) {
                if (((Map) obj).containsKey(key)) {
                    res.add(((Map) obj).get(key));
                }
            }
        }
        return res;
    }

    public void loadModel(Object modelYAML, ObjectModelImpl modelOM) {
        if (modelYAML instanceof List) {
            //load imports
            imports = new LinkedHashMap<String, String>();
            Object importsYAML = collectElement((List) modelYAML, IMPORTS);
            if(importsYAML instanceof List)
            {
                for(Object importYAML : (List) importsYAML)
                {
                    imports.put(afterChar(String.valueOf(importYAML),'.'),String.valueOf(importYAML));
                    log("import="+String.valueOf(importYAML));
                }
            }

            //load package
            Object packageYAML = collectElement((List) modelYAML, PACKAGE);
            if (packageYAML == null) {
                packageOM = "";
            } else {
                packageOM = String.valueOf(packageYAML);
            }
            log("package="+packageOM);

            Object nameYAML = collectElement((List) modelYAML, NAME);
            if (nameYAML == null) {
                ;
            } else {
                modelOM.setName(String.valueOf(nameYAML));
                log("name="+String.valueOf(nameYAML));
            }

            Object versionYAML = collectElement((List) modelYAML, VERSION);
            if (versionYAML == null) {
                ;
            } else {
                modelOM.setVersion(String.valueOf(versionYAML));
                log("version="+String.valueOf(versionYAML));
            }

            Object tagValuesYAML = collectElement((List) modelYAML, TAG_VALUES);
            if (tagValuesYAML instanceof Map) {
                for (Object entry : ((Map) tagValuesYAML).entrySet()) {
                    if (entry instanceof Map.Entry) {
                        modelOM.addTagValue(String.valueOf(((Map.Entry) entry).getKey()), String.valueOf(((Map.Entry) entry).getValue()));
                        log("tagValue=[key="+String.valueOf(((Map.Entry) entry).getKey())+" value="+String.valueOf(((Map.Entry) entry).getValue())+"]");
                    }
                }
            }

            //test imports objets modele
            List<Object> objectsYAML = collectAllElements((List) modelYAML, CLASS);
            objectsYAML.addAll(collectAllElements((List) modelYAML, INTERFACE));
            objectsYAML.addAll(collectAllElements((List) modelYAML, CLASS_ASSOCIATION));
            objectsYAML.addAll(collectAllElements((List) modelYAML, ENUMERATION));
            for(Object objectYAML : objectsYAML)
            {
                if(objectYAML instanceof List)
                {
                    Object importNameYAML = collectElement((List) objectYAML, NAME);
                    //Object importPackageYAML = collectElement((List) objectYAML, PACKAGE);
                    imports.put(String.valueOf(importNameYAML),packageOM+"."+String.valueOf(importNameYAML));
                    log("import="+imports.get(String.valueOf(importNameYAML)));
                    log(imports.toString());
                }
            }
            //test imports objets modele

            List classesYAML = collectAllElements((List) modelYAML, CLASS);
            for (Object classYAML : classesYAML) {
                ObjectModelClassImpl classOM = new ObjectModelClassImpl();
                loadClass(classYAML, classOM);
                modelOM.addClass(classOM);
                log("class="+classOM.getName()+" loaded");
            }

            List interfacesYAML = collectAllElements((List) modelYAML, INTERFACE);
            for (Object interfaceYAML : interfacesYAML) {
                ObjectModelInterfaceImpl interfaceOM = new ObjectModelInterfaceImpl();
                loadInterface(interfaceYAML, interfaceOM);
                modelOM.addInterface(interfaceOM);
                log("interface="+interfaceOM.getName()+" loaded");
            }

            List classesAssociationYAML = collectAllElements((List) modelYAML, CLASS_ASSOCIATION);
            for (Object classAssociationYAML : classesAssociationYAML) {
                ObjectModelAssociationClassImpl classAssociationOM = new ObjectModelAssociationClassImpl();
                loadAssociationClass(classAssociationYAML, classAssociationOM);
                modelOM.addAssociationClass(classAssociationOM);
                log("classAssociation="+classAssociationOM.getName()+" loaded");
            }

            List enumerationsYAML = collectAllElements((List) modelYAML, ENUMERATION);
            for (Object enumerationYAML : enumerationsYAML) {
                ObjectModelEnumerationImpl enumerationOM = new ObjectModelEnumerationImpl();
                loadEnumeration(enumerationYAML, enumerationOM);
                modelOM.addEnumeration(enumerationOM);
                log("enumeration="+enumerationOM.getName()+" loaded");
            }
        }
    }

    public void loadElement(Object elementYAML, ObjectModelElementImpl elementOM) {
        if (elementYAML instanceof List) {
            //TODO: setDeclaringElement
            //TODO: setObjectModelImpl
            //TODO: addModifier

            Object nameYAML = collectElement((List) elementYAML, NAME);
            if (nameYAML == null) {
                ;
            } else {
                elementOM.setName(String.valueOf(nameYAML));
                log("name="+String.valueOf(nameYAML));
            }

            Object staticYAML = collectElement((List) elementYAML, STATIC);
            if (staticYAML == null) {
                ;
            } else {
                elementOM.setStatic(Boolean.valueOf(String.valueOf(staticYAML)));
                log("static="+(Boolean.valueOf(String.valueOf(staticYAML))));
            }

            Object documentationYAML = collectElement((List) elementYAML, DOCUMENTATION);
            if (documentationYAML == null) {
                ;
            } else {
                elementOM.setDocumentation(String.valueOf(documentationYAML));
                log("documentation="+String.valueOf(documentationYAML));
            }

            Object tagValuesYAML = collectElement((List) elementYAML, TAG_VALUES);
            if (tagValuesYAML instanceof Map) {
                for (Object entry : ((Map) tagValuesYAML).entrySet()) {
                    if (entry instanceof Map.Entry) {
                        elementOM.addTagValue(String.valueOf(((Map.Entry) entry).getKey()), String.valueOf(((Map.Entry) entry).getValue()));
                        log("tagValue=[key="+String.valueOf(((Map.Entry) entry).getKey())+" value="+String.valueOf(((Map.Entry) entry).getValue())+"]");
                    }
                }
            }

            Object commentsYAML = collectElement((List) elementYAML, COMMENTS);
            if (commentsYAML instanceof List) {
                for (Object comment : (List) commentsYAML) {
                    elementOM.addComment(String.valueOf(comment));
                    log("comment="+String.valueOf(comment));
                }
            }

            Object stereotypesYAML = collectElement((List) elementYAML, STEREOTYPES);
            {
                if (stereotypesYAML instanceof List) {
                    for (Object stereotype : (List) stereotypesYAML) {
                        ObjectModelImplRef stereotypeOM = new ObjectModelImplRef();
                        stereotypeOM.setName(String.valueOf(stereotype));
                        elementOM.addStereotype(stereotypeOM);
                        log("stereotype="+String.valueOf(stereotype));
                    }
                }
            }
        }
    }

    public void loadClassifier(Object classifierYAML, ObjectModelClassifierImpl classifierOM) {
        if (classifierYAML instanceof List) {
            //TODO: addDependency

            loadElement(classifierYAML, classifierOM);

            Object packageYAML = collectElement((List) classifierYAML, PACKAGE);
            if (packageYAML == null) {
                classifierOM.setPackage(packageOM);
            } else {
                classifierOM.setPackage(String.valueOf(packageYAML));
                log("package="+String.valueOf(packageYAML));
            }

            Object externYAML = collectElement((List) classifierYAML, EXTERN);
            if (externYAML == null) {
                ;
            } else {
                classifierOM.setExtern(Boolean.valueOf(String.valueOf(externYAML)));
                log("extern="+String.valueOf(externYAML));
            }

            Object innerYAML = collectElement((List) classifierYAML, INNER);
            if (innerYAML == null) {
                ;
            } else {
                classifierOM.setInner(Boolean.valueOf(String.valueOf(innerYAML)));
                log("inner="+String.valueOf(innerYAML));
            }

            Object typeYAML = collectElement((List) classifierYAML, TYPE);
            if (typeYAML == null) {
                ;
            } else {
                classifierOM.setType(String.valueOf(typeYAML));
                log("type="+String.valueOf(typeYAML));
            }

            List attributesYAML = collectAllElements((List) classifierYAML, ATTRIBUTE);
            for (Object attributeYAML : attributesYAML) {
                ObjectModelAttributeImpl attributeOM = new ObjectModelAttributeImpl();
                loadAttribute(attributeYAML, attributeOM);
                classifierOM.addAttribute(attributeOM);
                log("attribute="+attributeOM.getType()+" loaded");
            }

            List operationsYAML = collectAllElements((List) classifierYAML, OPERATION);
            for (Object operationYAML : operationsYAML) {
                ObjectModelOperationImpl operationOM = new ObjectModelOperationImpl();
                loadOperation(operationYAML, operationOM);
                classifierOM.addOperation(operationOM);
                log("operation="+operationOM.getName()+" loaded");
            }

            Object superInterfacesYAML = collectElement((List) classifierYAML, SUPER_INTERFACES);
            if (superInterfacesYAML instanceof List) {
                for (Object superInterface : (List) superInterfacesYAML) {
                    ObjectModelImplRef superInterfaceOM = new ObjectModelImplRef();
                    superInterfaceOM.setName(String.valueOf(superInterface));
                    classifierOM.addInterface(superInterfaceOM);
                    log("superInterface="+String.valueOf(superInterface));
                }
            }
        }
    }

    public void loadClass(Object classYAML, ObjectModelClassImpl classOM) {
        if (classYAML instanceof List) {
            //TODO: addInnerClassifier

            loadClassifier(classYAML, classOM);

            Object abstractYAML = collectElement((List) classYAML, ABSTRACT);
            if (abstractYAML == null) {
                ;
            } else {
                classOM.setAbstract(Boolean.valueOf(String.valueOf(abstractYAML)));
                log("abstract="+String.valueOf(abstractYAML));
            }

            Object superClassesYAML = collectElement((List) classYAML, SUPER_CLASSES);
            if (superClassesYAML instanceof List) {
                for (Object superClass : (List) superClassesYAML) {
                    ObjectModelImplSuperClassRef superClassOM = new ObjectModelImplSuperClassRef();
                    //ObjectModelImplRef superClassOM = new ObjectModelImplRef();
                    superClassOM.setName(String.valueOf(superClass));
                    classOM.addSuperclass(superClassOM);
                    log("superClass="+String.valueOf(superClass));
                }
            }
        }
    }

    public void loadInterface(Object interfaceYAML, ObjectModelInterfaceImpl interfaceOM) {
        if (interfaceYAML instanceof List) {
            loadClassifier(interfaceYAML, interfaceOM);
        }
    }

    //TODO: gerer participants sous forme de liste -ou pas ? "attribute" utile ?-
    public void loadAssociationClass(Object associationClassYAML, ObjectModelAssociationClassImpl associationClassOM) {
        if (associationClassYAML instanceof List) {
            loadClass(associationClassYAML, associationClassOM);

            List participantsYAML = collectAllElements((List) associationClassYAML, PARTICIPANT);
            for (Object participantYAML : participantsYAML) {
                if (participantYAML instanceof List) {

                    ObjectModeImplAssociationClassParticipant participantOM = new ObjectModeImplAssociationClassParticipant();
                    participantOM.setAssociationClass(associationClassOM);

                    //le nom de la classe sans oublier la package devant
                    Object nameYAML = collectElement((List) participantYAML, NAME);
                    if (nameYAML == null) {
                        ;
                    } else {
                        String name = String.valueOf(nameYAML);
                        if (imports.containsKey(name)) {
                            participantOM.setName(imports.get(name));
                        } else {
                            participantOM.setName(name);
                        }
                        log("name="+name);
                    }

                    Object attributeYAML = collectElement((List) participantYAML, ATTRIBUTE);
                    if (attributeYAML == null) {
                        ;
                    } else {
                        participantOM.setAttribute(String.valueOf(attributeYAML));
                        log("attribute="+String.valueOf(attributeYAML));
                    }
                    log("participant="+String.valueOf(nameYAML)+" loaded");
                    associationClassOM.addParticipant(participantOM);
                }
            }
        }
    }

    public void loadEnumeration(Object enumerationYAML, ObjectModelEnumerationImpl enumerationOM) {
        if (enumerationYAML instanceof List) {
            loadElement(enumerationYAML, enumerationOM);

            Object literalsYAML = collectElement((List) enumerationYAML, LITERAL);
            if (literalsYAML instanceof List) {
                for (Object literalYAML : (List) literalsYAML) {
                    ObjectModelImplRef literalOM = new ObjectModelImplRef();
                    literalOM.setName(String.valueOf(literalYAML));
                    enumerationOM.addLiteral(literalOM);
                    log("literal="+String.valueOf(literalsYAML));
                }
            }

            Object packageYAML = collectElement((List) enumerationYAML, PACKAGE);
            if (packageYAML == null) {
                enumerationOM.setPackage(packageOM);
            } else {
                enumerationOM.setPackage(String.valueOf(packageYAML));
                log("package="+String.valueOf(packageYAML));
            }
        }
    }

    public void loadParameter(Object parameterYAML, ObjectModelParameterImpl parameterOM) {
        if (parameterYAML instanceof List) {
            loadElement(parameterYAML, parameterOM);

            Object orderingYAML = collectElement((List) parameterYAML, ORDERING);
            if (orderingYAML == null) {
                ;
            } else {
                parameterOM.setOrdering(String.valueOf(orderingYAML));
                log("ordering="+String.valueOf(orderingYAML));
            }

            Object typeYAML = collectElement((List) parameterYAML, TYPE);
            if (typeYAML == null) {
                ;//lever exception
            } else {
                String type = String.valueOf(typeYAML);
                if (imports.containsKey(type)) {
                    parameterOM.setType(imports.get(type));
                } else {
                    parameterOM.setType(type);
                }
                log("type="+parameterOM.getType());
            }

            Object defaultValueYAML = collectElement((List) parameterYAML, DEFAULT_VALUE);
            if (defaultValueYAML == null) {
                ;
            } else {
                parameterOM.setDefaultValue(String.valueOf(defaultValueYAML));
                log("defaultValue="+String.valueOf(defaultValueYAML));
            }

            Object minMultiplicityYAML = collectElement((List) parameterYAML, MIN_MULTIPLICITY);
            if (minMultiplicityYAML == null) {
                ;//deja instancié à 1
            } else {
                parameterOM.setMinMultiplicity(Integer.valueOf(String.valueOf(minMultiplicityYAML)));
                log("minMultiplicity="+Integer.valueOf(String.valueOf(minMultiplicityYAML)));
            }

            Object maxMultiplicityYAML = collectElement((List) parameterYAML, MAX_MULTIPLICITY);
            if (maxMultiplicityYAML == null) {
                ;
            } else {
                parameterOM.setMaxMultiplicity(Integer.valueOf(String.valueOf(maxMultiplicityYAML)));
                log("maxMultiplicity="+String.valueOf(maxMultiplicityYAML));
            }

            Object orderedYAML = collectElement((List) parameterYAML, ORDERED);
            if (orderedYAML == null) {
                ;
            } else {
                parameterOM.setOrdered(Boolean.valueOf(String.valueOf(orderedYAML)));
                log("ordered="+Boolean.valueOf(String.valueOf(orderedYAML)));
            }

            Object uniqueYAML = collectElement((List) parameterYAML, UNIQUE);
            if (uniqueYAML == null) {
                ;
            } else {
                parameterOM.setUnique(Boolean.valueOf(String.valueOf(uniqueYAML)));
                log("unique="+Boolean.valueOf(String.valueOf(uniqueYAML)));
            }
        }
    }

    public void loadAttribute(Object attributeYAML, ObjectModelAttributeImpl attributeOM) {
        if (attributeYAML instanceof List) {
            loadParameter(attributeYAML, attributeOM);

            Object navigableYAML = collectElement((List) attributeYAML, NAVIGABLE);
            if (navigableYAML == null) {
                ;
            } else {
                attributeOM.setNavigable(Boolean.valueOf(String.valueOf(navigableYAML)));
                log("navigable="+Boolean.valueOf(String.valueOf(navigableYAML)));
            }

            Object associationTypeYAML = collectElement((List) attributeYAML, ASSOCIATION_TYPE);
            if (associationTypeYAML == null) {
                ;//attributeOM.setAssociationType(ObjectModelAttributeImpl.ATTRIBUTE_TYPE_COMPOSITE);
            } else {
                attributeOM.setAssociationType(String.valueOf(associationTypeYAML));
                log("associationType="+String.valueOf(associationTypeYAML));
            }

            Object finalYAML = collectElement((List) attributeYAML, FINAL);
            if (finalYAML == null) {
                ;
            } else {
                attributeOM.setFinal(Boolean.valueOf(String.valueOf(finalYAML)));
                log("final="+Boolean.valueOf(String.valueOf(finalYAML)));
            }

            Object staticYAML = collectElement((List) attributeYAML, STATIC);
            if (staticYAML == null) {
                ;
            } else {
                attributeOM.setStatic(Boolean.valueOf(String.valueOf(staticYAML)));
                log("static="+Boolean.valueOf(String.valueOf(staticYAML)));
            }

            Object associationClassNameYAML = collectElement((List) attributeYAML, ASSOCIATION_CLASS_NAME);
            if (associationClassNameYAML == null) {
                ;
            } else {
                String associationClassName = String.valueOf(associationClassNameYAML);
                if (imports.containsKey(associationClassName)) {
                    attributeOM.setAssociationClassName(imports.get(associationClassName));
                } else {
                    attributeOM.setAssociationClassName(associationClassName);
                }
                log("associationClassName="+associationClassName);
            }

            Object reverseAttributeNameYAML = collectElement((List) attributeYAML, REVERSE_ATTRIBUTE_NAME);
            if (reverseAttributeNameYAML == null) {
                ;// attributeOM.setReverseAttributeName("");//test
            } else {
                attributeOM.setReverseAttributeName(String.valueOf(reverseAttributeNameYAML));
                log("reverseAttributeName="+String.valueOf(reverseAttributeNameYAML));
            }

            Object reverseMaxMultiplicityYAML = collectElement((List) attributeYAML, REVERSE_MAX_MULTIPLICITY);
            if (reverseMaxMultiplicityYAML == null) {
                ;//instancié à -1
            } else {
                attributeOM.setReverseMaxMultiplicity(Integer.valueOf(String.valueOf(reverseMaxMultiplicityYAML)));
                log("reverseMaxMultiplicity="+Integer.valueOf(String.valueOf(reverseMaxMultiplicityYAML)));
            }


            Object transientYAML = collectElement((List) attributeYAML, TRANSIENT);
            if (transientYAML == null) {
                ;
            } else {
                attributeOM.setTransient(Boolean.valueOf(String.valueOf(transientYAML)));
                log("transient="+Boolean.valueOf(String.valueOf(transientYAML)));
            }

            Object visibilityYAML = collectElement((List) attributeYAML, VISIBILITY);
            if (visibilityYAML == null) {
                ;
            } else {
                attributeOM.setVisibility(String.valueOf(visibilityYAML));
                log("visibility="+String.valueOf(visibilityYAML));
            }
        }
    }

    public void loadOperation(Object operationYAML, ObjectModelOperationImpl operationOM) {
        if (operationYAML instanceof List) {
            //TODO: addExceptionParameter

            loadElement(operationYAML, operationOM);

            Object abstractYAML = collectElement((List) operationYAML, ABSTRACT);
            if (abstractYAML == null) {
                ;
            } else {
                operationOM.setAbstract(Boolean.valueOf(String.valueOf(abstractYAML)));
                log("abstract="+Boolean.valueOf(String.valueOf(abstractYAML)));
            }

            Object visibilityYAML = collectElement((List) operationYAML, VISIBILITY);
            if (visibilityYAML == null) {
                ;
            } else {
                operationOM.setVisibility(String.valueOf(visibilityYAML));
                log("visibility="+String.valueOf(visibilityYAML));
            }

            Object returnParameterYAML = collectElement((List) operationYAML, RETURN_PARAMETER);
            if (returnParameterYAML == null) {
                ;
            } else {
                ObjectModelParameterImpl parameterOM = new ObjectModelParameterImpl();
                loadParameter(returnParameterYAML, parameterOM);
                operationOM.setReturnParameter(parameterOM);
                log("returnParameter="+parameterOM.getType()+" loaded");
            }

            Object bodyCodeYAML = collectElement((List) operationYAML, BODY_CODE);
            if (bodyCodeYAML == null) {
                ;
            } else {
                operationOM.setBodyCode(String.valueOf(bodyCodeYAML));
                log("bodyCode="+String.valueOf(bodyCodeYAML));
            }

            List parametersYAML = collectAllElements((List) operationYAML, PARAMETER);
            for (Object parameterYAML : parametersYAML) {
                ObjectModelParameterImpl parameterOM = new ObjectModelAttributeImpl();
                loadParameter(parameterYAML, parameterOM);
                operationOM.addParameter(parameterOM);
                log("parameter="+parameterOM.getType()+" loaded");
            }
        }
    }
}
